@@ -26,6 +26,8 @@
  *  Audacious or using our public API to be a derived work.
  */
 
+/* #define AUD_DEBUG 1 */
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -71,9 +73,10 @@
 
 #include "hook.h"
 
+#include "pluginenum.h"
+
 #include "playlist_evmessages.h"
 #include "playlist_evlisteners.h"
-#include "ui_skinned_playlist.h"
 
 typedef gint (*PlaylistCompareFunc) (PlaylistEntry * a, PlaylistEntry * b);
 typedef void (*PlaylistSaveFunc) (FILE * file);
@@ -153,6 +156,7 @@ static gboolean playlist_entry_get_info(PlaylistEntry * entry);
 
 static gint filter_by_extension(const gchar *filename);
 static gboolean is_http(const gchar *filename);
+static gboolean do_precheck(Playlist *playlist, const gchar *uri, ProbeResult **pr);
 
 static mowgli_heap_t *playlist_entry_heap = NULL;
 
@@ -218,15 +222,27 @@ playlist_entry_get_info(PlaylistEntry * entry)
 
     if (entry->decoder == NULL) {
         pr = input_check_file(entry->filename, FALSE);
-        if (pr)
+        if (pr != NULL)
             entry->decoder = pr->ip;
     }
 
     /* renew tuple if file mtime is newer than tuple mtime. */
     if (entry->tuple){
-        if (tuple_get_int(entry->tuple, FIELD_MTIME, NULL) == modtime)
+        if (tuple_get_int(entry->tuple, FIELD_MTIME, NULL) == modtime) {
+            if (pr != NULL) g_free(pr);
+            
+            if (entry->title_is_valid == FALSE) { /* update title even tuple is present and up to date --asphyx */
+                AUDDBG("updating title from actual tuple\n");
+                formatter = tuple_get_string(entry->tuple, FIELD_FORMATTER, NULL);
+                if (entry->title != NULL) g_free(entry->title);
+                entry->title = tuple_formatter_make_title_string(entry->tuple, formatter ?
+                                                  formatter : get_gentitle_format());
+                entry->title_is_valid = TRUE;
+                AUDDBG("new title: \"%s\"\n", entry->title);
+            }
+
             return TRUE;
-        else {
+        } else {
             mowgli_object_unref(entry->tuple);
             entry->tuple = NULL;
         }
@@ -235,10 +251,15 @@ playlist_entry_get_info(PlaylistEntry * entry)
     if (pr != NULL && pr->tuple != NULL)
         tuple = pr->tuple;
     else if (entry->decoder != NULL && entry->decoder->get_song_tuple != NULL)
+    {
+        plugin_set_current((Plugin *)(entry->decoder));
         tuple = entry->decoder->get_song_tuple(entry->filename);
+    }
 
-    if (tuple == NULL)
+    if (tuple == NULL) {
+        if (pr != NULL) g_free(pr);
         return FALSE;
+    }
 
     /* attach mtime */
     tuple_associate_int(tuple, FIELD_MTIME, NULL, modtime);
@@ -247,10 +268,11 @@ playlist_entry_get_info(PlaylistEntry * entry)
     formatter = tuple_get_string(tuple, FIELD_FORMATTER, NULL);
     entry->title = tuple_formatter_make_title_string(tuple, formatter ?
                                                   formatter : get_gentitle_format());
+    entry->title_is_valid = TRUE;
     entry->length = tuple_get_int(tuple, FIELD_LENGTH, NULL);
     entry->tuple = tuple;
 
-    g_free(pr);
+    if (pr != NULL) g_free(pr);
 
     return TRUE;
 }
@@ -656,11 +678,13 @@ playlist_delete(Playlist * playlist, gboolean crop)
 }
 
 static void
-__playlist_ins_with_info_tuple(Playlist * playlist,
-			       const gchar * filename,
-			       gint pos,
-			       Tuple *tuple,
-			       InputPlugin * dec)
+__playlist_ins_file(Playlist * playlist,
+                    const gchar * filename,
+                    gint pos,
+                    Tuple *tuple,
+                    const gchar *title, // may NULL
+                    gint len,
+                    InputPlugin * dec)
 {
     PlaylistEntry *entry;
     Tuple *parent_tuple = NULL;
@@ -689,16 +713,23 @@ __playlist_ins_with_info_tuple(Playlist * playlist,
              * to plugin, by passing the ?subtune suffix; this way we get
              * specific subtune information in the tuple, if available.
              */
+            plugin_set_current((Plugin *)dec);
             tuple = dec->get_song_tuple(filename_entry);
         } else
             filename_entry = g_strdup(filename);
         
-        
-        entry = playlist_entry_new(filename_entry,
-            tuple ? tuple_get_string(tuple, FIELD_TITLE, NULL) : NULL,
-            tuple ? tuple_get_int(tuple, FIELD_LENGTH, NULL) : -1, dec);
+        if(tuple) {
+            entry = playlist_entry_new(filename_entry,
+                                       tuple_get_string(tuple, FIELD_TITLE, NULL),
+                                       tuple_get_int(tuple, FIELD_LENGTH, NULL), dec);
+        }
+        else {
+            entry = playlist_entry_new(filename_entry, title, len, dec);
+        }
+
         g_free(filename_entry);
 
+
         PLAYLIST_LOCK(playlist);
         
         if (!playlist->tail)
@@ -726,8 +757,10 @@ __playlist_ins_with_info_tuple(Playlist * playlist,
         
         if (tuple != NULL) {
             const gchar *formatter = tuple_get_string(tuple, FIELD_FORMATTER, NULL);
+            g_free(entry->title);
             entry->title = tuple_formatter_make_title_string(tuple,
                 formatter ? formatter : get_gentitle_format());
+            entry->title_is_valid = TRUE;
             entry->length = tuple_get_int(tuple, FIELD_LENGTH, NULL);
             entry->tuple = tuple;
         }
@@ -757,8 +790,7 @@ playlist_ins(Playlist * playlist, const gchar * filename, gint pos)
     ProbeResult *pr = NULL;
     InputPlugin *dec = NULL;
     Tuple *tuple = NULL;
-    gint ext_flag;
-    gboolean http_flag;
+    gboolean http_flag = is_http(filename);
 
     g_return_val_if_fail(playlist != NULL, FALSE);
     g_return_val_if_fail(filename != NULL, FALSE);
@@ -771,44 +803,25 @@ playlist_ins(Playlist * playlist, const gchar * filename, gint pos)
         return TRUE;
     }
 
-    ext_flag = filter_by_extension(filename);
-    http_flag = is_http(filename);
-
-    /* playlist file or remote uri */
-    if (playlist->loading_playlist == TRUE || http_flag == TRUE) {
-        dec = NULL;
-    }
-
-    /* local file and on-demand probing is on */
-    else if (cfg.playlist_detect == TRUE && ext_flag != EXT_HAVE_SUBTUNE && ext_flag != EXT_CUSTOM) {
-        dec = NULL;
-        if(cfg.use_extension_probing && ext_flag == EXT_FALSE)
-            return FALSE;
-    }
-
-    /* find decorder for local file */
-    else {
-        pr = input_check_file(filename, TRUE);
-
-        if (pr) {
+    if(do_precheck(playlist, filename, &pr)) {
+        if(pr) {
             dec = pr->ip;
             tuple = pr->tuple;
         }
-        g_free(pr);
-    }
+        /* add filename to playlist */
+        if (cfg.playlist_detect == TRUE ||
+            playlist->loading_playlist == TRUE ||
+            (playlist->loading_playlist == FALSE && dec != NULL) ||
+            (playlist->loading_playlist == FALSE && !is_playlist_name(filename) && http_flag) ) {
 
-    /* add filename to playlist */
-    if (cfg.playlist_detect == TRUE ||
-        playlist->loading_playlist == TRUE ||
-       (playlist->loading_playlist == FALSE && dec != NULL) ||
-       (playlist->loading_playlist == FALSE && !is_playlist_name(filename) && http_flag) ) {
+            __playlist_ins_file(playlist, filename, pos, tuple, NULL, -1, dec);
 
-        __playlist_ins_with_info_tuple(playlist, filename, pos, tuple, dec);
-
-        playlist_generate_shuffle_list(playlist);
-        playlistwin_update_list(playlist);
-        playlist_manager_update();
-        return TRUE;
+            g_free(pr);
+            playlist_generate_shuffle_list(playlist);
+            playlistwin_update_list(playlist);
+            playlist_manager_update();
+            return TRUE;
+        }
     }
 
     /* Some files (typically produced by some cgi-scripts) don't have
@@ -972,6 +985,8 @@ playlist_dir_find_files(const gchar * path,
             if(cfg.use_extension_probing) {
                 if(ext_flag == EXT_TRUE)
                     list = g_list_prepend(list, filename);
+                else // ext_flag == EXT_FALSE => extension isn't known
+                    g_free(filename);
             }
             else
                 list = g_list_prepend(list, filename);
@@ -1104,7 +1119,7 @@ playlist_ins_url(Playlist * playlist, const gchar * string,
     }
 
     playlist_recalc_total_time(playlist);
-    PLAYLIST_INCR_SERIAL(playlist); //probably necessary because there is no underlying __playlist_ins --yaz
+    PLAYLIST_INCR_SERIAL(playlist);
     playlist_generate_shuffle_list(playlist);
     playlistwin_update_list(playlist);
 
@@ -1113,20 +1128,20 @@ playlist_ins_url(Playlist * playlist, const gchar * string,
     return entries;
 }
 
+/* set info for current song. */
 void
-playlist_set_info_old_abi(const gchar * title, gint length, gint rate,
+playlist_set_info(Playlist * playlist, const gchar * title, gint length, gint rate,
                           gint freq, gint nch)
 {
-    Playlist *playlist = playlist_get_active();
     PlaylistEventInfoChange *msg;
     gchar *text;
 
+    g_return_if_fail(playlist != NULL);
+
     if(length == -1) {
         event_queue("hide seekbar", (gpointer)0xdeadbeef); // event_queue hates NULL --yaz
     }
 
-    g_return_if_fail(playlist != NULL);
-
     if (playlist->position) {
         g_free(playlist->position->title);
         playlist->position->title = g_strdup(title);
@@ -1156,25 +1171,13 @@ playlist_set_info_old_abi(const gchar * title, gint length, gint rate,
         hook_call( "playlist set info" , playlist->position );
 }
 
+/* wrapper for playlist_set_info. this function is called by input plugins. */
 void
-playlist_set_info(Playlist * playlist, const gchar * title, gint length, gint rate,
-                  gint freq, gint nch)
+playlist_set_info_old_abi(const gchar * title, gint length, gint rate,
+                          gint freq, gint nch)
 {
-    g_return_if_fail(playlist != NULL);
-
-    if (playlist->position) {
-        g_free(playlist->position->title);
-        playlist->position->title = g_strdup(title);
-        playlist->position->length = length;
-    }
-
-    playlist_recalc_total_time(playlist);
-    PLAYLIST_INCR_SERIAL(playlist); //tentative --yaz
-
-    mainwin_set_song_info(rate, freq, nch);
-
-    if (playlist->position)
-        hook_call( "playlist set info" , playlist->position );
+    Playlist *playlist = playlist_get_active();
+    playlist_set_info(playlist, title, length, rate, freq, nch);
 }
 
 void
@@ -1186,7 +1189,7 @@ playlist_check_pos_current(Playlist *playlist)
         return;
 
     PLAYLIST_LOCK(playlist);
-    if (!playlist->position || !playlistwin_list) {
+    if (!playlist->position) {
         PLAYLIST_UNLOCK(playlist);
         return;
     }
@@ -1198,9 +1201,8 @@ playlist_check_pos_current(Playlist *playlist)
         return;
     }
 
-    bottom = MAX(0, playlist_get_length(playlist) -
-                 UI_SKINNED_PLAYLIST(playlistwin_list)->num_visible);
-    row = CLAMP(pos - UI_SKINNED_PLAYLIST(playlistwin_list)->num_visible / 2, 0, bottom);
+    bottom = MAX(0, playlist_get_length(playlist) - playlistwin_list_get_visible_count());
+    row = CLAMP(pos - playlistwin_list_get_visible_count() / 2, 0, bottom);
     PLAYLIST_UNLOCK(playlist);
     playlistwin_set_toprow(row);
     g_cond_signal(cond_scan);
@@ -1708,100 +1710,75 @@ playlist_load(Playlist * playlist, const gchar * filename)
 
 void
 playlist_load_ins_file(Playlist *playlist,
-                       const gchar * filename_p,
+                       const gchar * uri,
                        const gchar * playlist_name, gint pos,
                        const gchar * title, gint len)
 {
-    Tuple *tuple;
+    ProbeResult *pr = NULL;
 
-    if (vfs_is_remote(filename_p)) {
-      tuple = tuple_new();
-      tuple_associate_string(tuple, FIELD_FILE_NAME, NULL, filename_p);        
-    } else {
-      tuple = tuple_new_from_filename(filename_p);
+    g_return_if_fail(uri != NULL);
+    g_return_if_fail(playlist_name != NULL);
+    g_return_if_fail(playlist != NULL);
+
+    if(do_precheck(playlist, uri, &pr)) {
+        __playlist_ins_file(playlist, uri, pos, NULL, title, len, pr ? pr->ip : NULL);
     }
-
-    tuple_associate_string(tuple, FIELD_TITLE, NULL, title);
-    tuple_associate_int(tuple, FIELD_LENGTH, NULL, len);
-    tuple_associate_int(tuple, FIELD_MTIME, NULL, -1); // invalidate to make tuple renew
-
-    playlist_load_ins_file_tuple(playlist, filename_p, playlist_name, pos, tuple);
+    g_free(pr);
 }
 
 void
 playlist_load_ins_file_tuple(Playlist * playlist,
-                             const gchar * filename_p,      //filename to add
+                             const gchar * uri,
                              const gchar * playlist_name,   //path of playlist file itself
                              gint pos,
                              Tuple *tuple)
 {
-    gchar *filename;
-    gchar *tmp, *path;
     ProbeResult *pr = NULL;		/* for decoder cache */
-    gchar *uri = NULL;
 
-
-    g_return_if_fail(filename_p != NULL);
+    g_return_if_fail(uri != NULL);
     g_return_if_fail(playlist_name != NULL);
     g_return_if_fail(playlist != NULL);
 
-    filename = g_strchug(g_strdup(filename_p));
-
-
-    /* convert backslash to slash */
-    if (cfg.convert_slash)
-        while ((tmp = strchr(filename, '\\')) != NULL)
-            *tmp = '/';
-
-
-    /* make full path uri here */
-    // case 1: filename is raw full path or uri
-    if (filename[0] == '/' || strstr(filename, "://")) {
-        uri = g_filename_to_uri(filename, NULL, NULL);
-        if(!uri) {
-            uri = g_strdup(filename);
-        }
-        g_free(filename);
-    }
-    // case 2: filename is not raw full path nor uri, playlist path is full path
-    // make full path by replacing last part of playlist path with filename. (using g_build_filename)
-    else if (playlist_name[0] == '/' || strstr(playlist_name, "://")) {
-        path = g_strdup(playlist_name);
-        tmp = strrchr(path, '/'); *tmp = '\0';
-        tmp = g_build_filename(path, filename, NULL);
-        uri = g_filename_to_uri(tmp, NULL, NULL);
-        g_free(tmp); g_free(filename);
-    }
-    // case 3: filename is not raw full path nor uri, playlist path is not full path
-    // just abort.
-    else {
-        g_free(filename);
-        return;
+    if(do_precheck(playlist, uri, &pr)) {
+        __playlist_ins_file(playlist, uri, pos, tuple, NULL, -1, pr ? pr->ip : NULL);
     }
+    g_free(pr);
 
+}
 
-    /* apply pre check and add to playlist */
+static gboolean
+do_precheck(Playlist *playlist, const gchar *uri, ProbeResult **pr)
+{
     gint ext_flag = filter_by_extension(uri);
     gboolean http_flag = is_http(uri);
+    gboolean rv = FALSE;
 
     /* playlist file or remote uri */
     if ((playlist->loading_playlist == TRUE && ext_flag != EXT_HAVE_SUBTUNE ) || http_flag == TRUE) {
         pr = NULL;
+        rv = TRUE;
     }
     /* local file and on-demand probing is on */
     else if (cfg.playlist_detect == TRUE && ext_flag != EXT_HAVE_SUBTUNE && ext_flag != EXT_CUSTOM) {
-        pr = NULL;
-        if(cfg.use_extension_probing && ext_flag == EXT_FALSE)
-            return;
+        if(cfg.use_extension_probing && ext_flag == EXT_FALSE) {
+            AUDDBG("reject %s\n", uri);
+            rv = FALSE;
+        }
+        else {
+            pr = NULL;
+            rv = TRUE;
+        }
     }
     /* find decorder for local file */
     else {
-        pr = input_check_file(uri, TRUE);
+        *pr = input_check_file(uri, TRUE);
+        if(*pr) {
+            AUDDBG("got pr\n");
+            rv = TRUE;
+        }
     }
 
-    __playlist_ins_with_info_tuple(playlist, uri, pos, tuple, pr ? pr->ip : NULL);
-    g_free(pr);
-
+    return rv;
 }
 
 static guint
@@ -2083,6 +2060,7 @@ playlist_compare_title(PlaylistEntry * a,
 
     if (a->tuple != NULL)
         a_title = tuple_get_string(a->tuple, FIELD_TITLE, NULL);
+
     if (b->tuple != NULL)
         b_title = tuple_get_string(b->tuple, FIELD_TITLE, NULL);
 
@@ -2112,12 +2090,20 @@ playlist_compare_artist(PlaylistEntry * a,
 
     if (a->tuple != NULL) {
         a_artist = tuple_get_string(a->tuple, FIELD_ARTIST, NULL);
+
+        if (a_artist == NULL)
+            return 0;
+
         if (str_has_prefix_nocase(a_artist, "the "))
             a_artist += 4;
     }
 
     if (b->tuple != NULL) {
         b_artist = tuple_get_string(b->tuple, FIELD_ARTIST, NULL);
+
+        if (b_artist == NULL)
+            return 0;
+
         if (str_has_prefix_nocase(b_artist, "the "))
             b_artist += 4;
     }
@@ -2472,44 +2458,61 @@ playlist_fileinfo(Playlist *playlist, guint pos)
 
     PLAYLIST_UNLOCK(playlist);
 
-    if (entry->tuple)
-        mtime = tuple_get_int(entry->tuple, FIELD_MTIME, NULL);
-    else
-        mtime = 0;
-
-    /* No tuple? Try to set this entry up properly. --nenolod */
-    if (entry->tuple == NULL || mtime == -1 ||
-        mtime == 0 || mtime != playlist_get_mtime(entry->filename))
+    if (entry->decoder == NULL)
     {
-        playlist_entry_get_info(entry);
-        tuple = entry->tuple;
+        pr = input_check_file(entry->filename, FALSE); /* try to find a decoder */
+        entry->decoder = pr ? pr->ip : NULL;
+
+        g_free(pr);
     }
 
-    if (tuple != NULL)
-    {
-        if (entry->decoder == NULL)
-        {
-            pr = input_check_file(entry->filename, FALSE); /* try to find a decoder */
-            entry->decoder = pr ? pr->ip : NULL;
+    /* plugin is capable to update tags. we need to bypass tuple cache. --eugene */
+    /* maybe code cleanup required... */
+    if (entry->decoder != NULL && entry->decoder->update_song_tuple != NULL &&
+        entry->decoder->file_info_box == NULL && path != NULL && !vfs_is_remote(path)) {
 
-            g_free(pr);
+        fileinfo_show_editor_for_path(path, entry->decoder);
+        g_free(path);
+
+    } else {
+
+        if (entry->tuple)
+            mtime = tuple_get_int(entry->tuple, FIELD_MTIME, NULL);
+        else
+            mtime = 0;
+
+        /* No tuple? Try to set this entry up properly. --nenolod */
+        if (entry->tuple == NULL || mtime == -1 ||
+            mtime == 0 || mtime != playlist_get_mtime(entry->filename))
+        {
+            playlist_entry_get_info(entry);
+            tuple = entry->tuple;
         }
 
-        if (entry->decoder != NULL && entry->decoder->file_info_box == NULL)
-            fileinfo_show_for_tuple(tuple);
-        else if (entry->decoder != NULL && entry->decoder->file_info_box != NULL)
-            entry->decoder->file_info_box(path);
-        else
-            fileinfo_show_for_path(path);
-        g_free(path);
-    }
-    else if (path != NULL)
-    {
-        if (entry != NULL && entry->decoder != NULL && entry->decoder->file_info_box != NULL)
-            entry->decoder->file_info_box(path);
-        else
-            fileinfo_show_for_path(path);
-        g_free(path);
+        if (tuple != NULL)
+        {
+            if (entry->decoder != NULL && entry->decoder->file_info_box == NULL)
+                fileinfo_show_for_tuple(tuple, FALSE);
+            else if (entry->decoder != NULL && entry->decoder->file_info_box != NULL)
+	    {
+                plugin_set_current((Plugin *)(entry->decoder));
+                entry->decoder->file_info_box(path);
+	    }
+            else
+                fileinfo_show_for_path(path);
+            g_free(path);
+        }
+        else if (path != NULL)
+        {
+            if (entry != NULL && entry->decoder != NULL && entry->decoder->file_info_box != NULL)
+	    {
+                plugin_set_current((Plugin *)(entry->decoder));
+                entry->decoder->file_info_box(path);
+	    }
+            else
+                fileinfo_show_for_path(path);
+            g_free(path);
+        }
     }
 }
 
@@ -2531,23 +2534,38 @@ playlist_fileinfo_current(Playlist *playlist)
 
     PLAYLIST_UNLOCK(playlist);
 
-    if (tuple != NULL)
-    {
-        if (playlist->position->decoder != NULL && playlist->position->decoder->file_info_box == NULL)
-            fileinfo_show_for_tuple(tuple);
-        else if (playlist->position->decoder != NULL && playlist->position->decoder->file_info_box != NULL)
-            playlist->position->decoder->file_info_box(path);
-        else
-            fileinfo_show_for_path(path);
-        g_free(path);
-    }
-    else if (path != NULL)
-    {
-        if (playlist->position != NULL && playlist->position->decoder != NULL && playlist->position->decoder->file_info_box != NULL)
-            playlist->position->decoder->file_info_box(path);
-        else
-            fileinfo_show_for_path(path);
+    if (playlist->position->decoder != NULL && playlist->position->decoder->update_song_tuple != NULL &&
+        playlist->position->decoder->file_info_box == NULL && path != NULL && !vfs_is_remote(path)) {
+
+        fileinfo_show_editor_for_path(path, playlist->position->decoder);
         g_free(path);
+
+    } else {
+
+        if (tuple != NULL)
+        {
+            if (playlist->position->decoder != NULL && playlist->position->decoder->file_info_box == NULL)
+                fileinfo_show_for_tuple(tuple, FALSE);
+            else if (playlist->position->decoder != NULL && playlist->position->decoder->file_info_box != NULL)
+	    {
+                plugin_set_current((Plugin *)(playlist->position->decoder));
+                playlist->position->decoder->file_info_box(path);
+	    }
+            else
+                fileinfo_show_for_path(path);
+            g_free(path);
+        }
+        else if (path != NULL)
+        {
+            if (playlist->position != NULL && playlist->position->decoder != NULL && playlist->position->decoder->file_info_box != NULL)
+	    {
+                plugin_set_current((Plugin *)(playlist->position->decoder));
+                playlist->position->decoder->file_info_box(path);
+	    }
+            else
+                fileinfo_show_for_path(path);
+            g_free(path);
+        }
     }
 }
 
@@ -2584,7 +2602,7 @@ playlist_get_info_func(gpointer arg)
 
                 if(playlist->attribute & PLAYLIST_STATIC || // live lock fix
                    (entry->tuple && tuple_get_int(entry->tuple, FIELD_LENGTH, NULL) > -1 &&
-                    tuple_get_int(entry->tuple, FIELD_MTIME, NULL) != -1)) {
+                    tuple_get_int(entry->tuple, FIELD_MTIME, NULL) != -1 && entry->title_is_valid)) {
                     update_playlistwin = TRUE;
                     continue;
                 }
@@ -2599,7 +2617,7 @@ playlist_get_info_func(gpointer arg)
                     tuple_get_int(entry->tuple, FIELD_LENGTH, NULL) > -1 &&
                     tuple_get_int(entry->tuple, FIELD_MTIME, NULL) != -1) {
                     update_playlistwin = TRUE;
-                    break;
+                    break; /* hmmm... --asphyx */
                 }
             }
             
@@ -2629,11 +2647,14 @@ playlist_get_info_func(gpointer arg)
 
                  if(playlist->attribute & PLAYLIST_STATIC ||
                    (entry->tuple && tuple_get_int(entry->tuple, FIELD_LENGTH, NULL) > -1 &&
-                    tuple_get_int(entry->tuple, FIELD_MTIME, NULL) != -1)) {
-                    update_playlistwin = TRUE;
+                    tuple_get_int(entry->tuple, FIELD_MTIME, NULL) != -1 && entry->title_is_valid)) {
                     continue;
                  }
 
+                 AUDDBG("len=%d mtime=%d\n",
+                        tuple_get_int(entry->tuple, FIELD_LENGTH, NULL),
+                        tuple_get_int(entry->tuple, FIELD_MTIME, NULL));
+
                  if (!playlist_entry_get_info(entry)) { 
                      if (g_list_index(playlist->entries, entry) == -1)
                         /* Entry disapeared while we
@@ -2648,6 +2669,7 @@ playlist_get_info_func(gpointer arg)
                  }
             }
         } // on_demand
+
         else if (cfg.get_info_on_demand &&
                  (!cfg.playlist_visible || cfg.playlist_shaded || !cfg.use_pl_metadata))
         {
@@ -2655,6 +2677,7 @@ playlist_get_info_func(gpointer arg)
             playlist_get_info_scan_active = FALSE;
             g_mutex_unlock(mutex_scan);
         }
+
         else /* not on_demand and not on_load...
                 NOTE: this shouldn't happen anymore, sanity check in bmp_config_load now */
         {
@@ -2664,7 +2687,9 @@ playlist_get_info_func(gpointer arg)
         }
 
         if (update_playlistwin) {
-            event_queue("playlistwin update list", playlist_get_active());
+            Playlist *playlist = playlist_get_active();
+            event_queue("playlistwin update list", playlist);
+            PLAYLIST_INCR_SERIAL(playlist);
             update_playlistwin = FALSE;
         }
 
@@ -2676,7 +2701,7 @@ playlist_get_info_func(gpointer arg)
         g_cond_wait(cond_scan, mutex_scan);
         g_mutex_unlock(mutex_scan);
 
-//        g_print("scanner invoked\n");
+//        AUDDBG("scanner invoked\n");
 
     } // while
 
@@ -2709,6 +2734,7 @@ playlist_stop_get_info_thread(void)
 void
 playlist_start_get_info_scan(void)
 {
+    AUDDBG("waking up scan thread\n");
     g_mutex_lock(mutex_scan);
     playlist_get_info_scan_active = TRUE;
     g_mutex_unlock(mutex_scan);
@@ -2716,6 +2742,23 @@ playlist_start_get_info_scan(void)
     g_cond_signal(cond_scan);
 }
 
+void
+playlist_update_all_titles(void) /* update titles after format changing --asphyx */
+{
+    PlaylistEntry *entry;
+    GList *node;
+    Playlist *playlist = playlist_get_active();
+    
+    AUDDBG("invalidating titles\n");
+    PLAYLIST_LOCK(playlist);
+    for (node = playlist->entries; node; node = g_list_next(node)) {
+        entry = node->data;
+        entry->title_is_valid = FALSE;
+    }
+    PLAYLIST_UNLOCK(playlist);
+    playlist_start_get_info_scan();
+}
+
 void
 playlist_remove_dead_files(Playlist *playlist)
 {
@@ -3297,7 +3340,9 @@ playlist_free(Playlist *playlist)
 {
     if (!playlist)
         return;
-    
+
+    if (playlist->filename)
+        g_free( playlist->filename );
     g_mutex_free( playlist->mutex );
     g_free( playlist );
     playlist = NULL; //XXX lead to crash? --yaz
@@ -3403,6 +3448,8 @@ filter_by_extension(const gchar *uri)
     gint rv;
     GList **lhandle, *node;
     InputPlugin *ip;
+
+    g_return_val_if_fail(uri != NULL, EXT_FALSE);
     
     /* Some URIs will end in ?<subsong> to determine the subsong requested. */
     tmp_uri = g_strdup(uri);
@@ -3425,11 +3472,12 @@ filter_by_extension(const gchar *uri)
 
 
     base = g_path_get_basename(filename);
+    g_free(filename);
     ext = strrchr(base, '.');
 
     if(!ext) {
         g_free(base);
-        return 0;
+        return EXT_FALSE;
     }
 
     lext = g_ascii_strdown(ext+1, -1);
